﻿// Author: Orlys
// Github: https://github.com/Orlys

namespace Yuuna.Contracts.Modules
{
    using System;
    using System.Collections.Immutable;
    using System.Runtime.Loader;
    using Yuuna.Contracts.Optimization;
    using Yuuna.Contracts.Patterns;
    using Yuuna.Contracts.Semantics;
    using Yuuna.Contracts.TextSegmention;
    public abstract class ModuleBase 
    {
        /// <summary>
        /// 中繼資料。
        /// </summary> 
        public ModuleMetadataAttribute Metadata { get; }

        public ModuleBase()
        {
            this.Metadata = ModuleMetadataAttribute.GetMetadata(this.GetType());
        }

        public bool Initialized { get; private set; }

        /// <summary>
        /// 初始化模組
        /// </summary>
        /// <param name="textSegmenter">分詞器</param>
        /// <param name="groupManager">群組管理</param>
        internal void Initialize(ITextSegmenter textSegmenter, IGroupManager groupManager, out IPatternSet patterns)
        {
            patterns = null;
            if (!this.Initialized)
            {
                try
                {
                    patterns = new PatternFactory(this);
                    this.BuildPatterns(groupManager, patterns as IPatternBuilder);
                    textSegmenter.Load(groupManager);
                    this.AfterInitialize();
                    this.Initialized = true;
                    this.Patterns = patterns;
                }
                catch (Exception e)
                {
                    throw new TypeInitializationException(this.GetType().FullName, e);
                }
            }
        }

        internal IPatternSet Patterns { get; private set; }

        /// <summary>
        /// 在初始化後引發。
        /// </summary>
        protected virtual void AfterInitialize()
        {
        }

        /// <summary>
        /// 建立模式規則。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="p"></param>
        protected abstract void BuildPatterns(IGroupManager g, IPatternBuilder p);
    }
}