﻿// Author: Orlys
// Github: https://github.com/Orlys

namespace Yuuna
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Reflection;
    using Yuuna.Contracts.Modules;
    using Yuuna.Contracts.Patterns;
    using Yuuna.Contracts.TextSegmention;
    using Yuuna.Contracts.Semantics;

    public sealed class ModuleCoupler : IDisposable
    {
        private WeakReference<ModuleBase> _module;
        private CollectibleLoader _loader;
        internal ModuleCoupler(IEnumerable<FileInfo> dlls)
        {
            this._loader = new CollectibleLoader();
            foreach (var dll in dlls)
            {
                using (var fs = dll.OpenRead())
                {
                    var asm = this._loader.LoadFromStream(fs);
                    foreach (var t in asm.GetTypes())
                    {
                        if (t.IsSubclassOf(typeof(ModuleBase)))
                        {
                            this._module = new WeakReference<ModuleBase>(Activator.CreateInstance(t) as ModuleBase);
                            if (this._module.TryGetTarget(out var m))
                            {
                                this.Metadata = m.Metadata;
                            }
                            break;
                        }
                    }

                }
            }
        }

        public IPatternSet Patterns { get; private set; }
        public IModuleMetadata Metadata { get; }

        public bool Initialize(ITextSegmenter s, IGroupManager g)
        {
            if (this._module is null)
                return false;

            if (this._module.TryGetTarget(out var m))
            {
                m.Initialize(s, g, out var v);
                this.Patterns = v;
                return true;
            }

            return false;
        }

        public void Dispose()
        {
            this._loader.Unload();
            this._loader = null;
        }
    }
}