﻿// Author: Orlys
// Github: https://github.com/Orlys

namespace Yuuna.Recognition.Speech
{
    using NAudio.Wave;

    using System;
    using System.Collections.Generic;
    using System.IO;

    using Yuuna.Contracts.Recorders;

    public class WaveRecorder : IRecorder
    {
        private readonly object _lock = new object();
        private readonly List<byte> _raw;
        private readonly WaveInEvent _wave;
        private volatile bool _started = false;

        public event Action<Stream> Completed;

        public WaveRecorder()
        {
            this._raw = new List<byte>(4096);
            this._wave = new WaveInEvent
            {
                WaveFormat = new WaveFormat(16000, 1)
            };

            this._wave.DataAvailable += this.OnDataAvailable;
        }

        public void Start()
        {
            lock (this._lock)
            {
                if (this._started)
                    return;
                this._raw.Clear();
                this._wave.StartRecording();
                this._started = true;
            }
        }

        public void Stop()
        {
            lock (this._lock)
            {
                if (!this._started)
                    return;
                this._wave.StopRecording();

                var array = this._raw.ToArray();
                var ms = new MemoryStream(array);

                var wavStream = new RawSourceWaveStream(ms, this._wave.WaveFormat);
                this.Completed.Invoke(wavStream);
                this._started = false;
            }
        }

        protected virtual void OnDataAvailable(object sender, WaveInEventArgs e)
        {
            lock (this._lock)
            {
                this._raw.AddRange(e.Buffer);
            }
        }
    }
}